package miscc;

import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import java.io.PrintWriter;

public class CFGDotPrinter {
    public void print(CFG cfg, PrintWriter writer) {
        writer.println("digraph \"" + cfg.name + "\" {");
        writer.println("node [ fontname=\"Courier\", shape=box ];");
        writer.println("edge [ fontname=\"Courier\" ];");

        List nodes = cfg.getNodes();
        Iterator nodesIt = nodes.iterator();
        while (nodesIt.hasNext()) {
            CFG.Node n = (CFG.Node)nodesIt.next();
            List outEdges = cfg.getEdgesFrom(n);
            Iterator outEdgeIt = outEdges.iterator();
            printNode(n, cfg, writer);
            while (outEdgeIt.hasNext()) {
                CFG.Edge e = (CFG.Edge)outEdgeIt.next();
                printEdge(e, cfg, writer);
            }
        }

        writer.println("}");
    }

    private void printNode(CFG.Node node, CFG cfg, PrintWriter w) {
        w.println(nodeName(node, cfg)
                  + " [ label=\"" + nodeLabel(node) + "\" ];");
    }

    private void printEdge(CFG.Edge edge, CFG cfg, PrintWriter w) {
        w.println(nodeName(edge.from, cfg)
                  + " -> "
                  + nodeName(edge.to, cfg)
                  + " [label=\"" + edge.branch + "\" ];");
    }

    private String nodeName(CFG.Node node, CFG cfg) {
	return cfg.name + "_" + node.label;
    }

    private String nodeLabel(CFG.Node node) {
        StringBuffer b = new StringBuffer();
        Iterator instIt = node.code.instructions().iterator();
        while (instIt.hasNext()) {
            Instruction inst = (Instruction)instIt.next();
            b.append(inst);
            b.append("\\l");
        }
        return b.toString();
    }
}
