/*  miscc -- a compiler for misc
 *    Support for code generation
 *
 *    Matthias Zenger, Michel Schinz, Philippe Altherr
 *    Erik Stenman
 *    $Id: Code.java,v 1.8 2004/04/28 11:53:02 schinz Exp $
 */

package miscc;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public class Code implements RISC {
    //########################################################################
    // Instance variables

    /** Already emitted instructions */
    private final List/*<Instruction>*/ code;

    //########################################################################
    // Constructor

    protected Code() {
        code = new ArrayList();
    }

    public int size() {
	int s = 0;
        Iterator instrIt = code.iterator();
        while (instrIt.hasNext())
            s += ((Instruction)instrIt.next()).size();
	return s;
    }

    //########################################################################
    // Methods - code emitting

    /** Return address of next instruction. */
    public int pc() {
        return WORD_SIZE * code.size();
    }

    public void emit(int opcode) {
        emit(opcode, Integer.MIN_VALUE);
    }

    public void emit(int opcode, int c) {
        emit(opcode, Integer.MIN_VALUE, c);
    }

    public void emit(int opcode, int a, int c) {
        emit(opcode, a, Integer.MIN_VALUE, c);
    }

    public void emit(int opcode, int a, int b, int c) {
	code.add(Instruction.make(opcode,a,b,c));
    }

    public void emit(int opcode, String c) {
	code.add(Instruction.make(opcode, Integer.MIN_VALUE, c));
    }

    public void emit(int opcode, int a, String c) {
	code.add(Instruction.make(opcode, a, c));
    }

    public void emit(Instruction i) {
	code.add(i);
    }

    public void addFirst(Instruction i) {
	code.add(0, i);
    }

    public void addLast(Instruction i) {
	code.add(i);
    }

    public void append(Code other) {
        code.addAll(other.code);
    }

    public List/*<Instruction>*/ instructions() {
        return Collections.unmodifiableList(code);
    }

    //########################################################################
    // Methods - program printing.

    /** Print program. */
    public void write(PrintWriter out) {
	int pos = 0;
        for (int i = 0; i < code.size(); i++) {
	    Instruction instr = (Instruction) code.get(i);
	    if (instr.opcode.code == RISC.LABEL) {
		out.print("          \t");
	    } else {
		String label = Integer.toString(pos);
		while (label.length() < 4) label = '0' + label;
		out.print("/* " + label + " */\t");
	    }
            out.print(instr);
	    pos += WORD_SIZE * instr.size();
            out.println();
        }
        out.flush();
    }

}
