package miscc;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;

public class Main {
    public static void main(String[] args) {
        try {
            InputStream in;
            PrintWriter out;

            if (args.length < 1)
                in = System.in;
            else
                in = new FileInputStream(args[0]);

            if (args.length < 2)
                out = new PrintWriter(System.out);
            else
                out = new PrintWriter(new FileOutputStream(args[1]));

            Parser parser = new Parser(in);
            Tree tree = parser.parse();
            in.close();

            Analyzer analyzer = new Analyzer();
            analyzer.analyze(tree);

            if (Report.errors == 0) {
                Generator generator = new Generator();
                CFG[] cfgs = generator.generate((Tree.Program)tree);

                CFGDotPrinter printer = new CFGDotPrinter();
                Spiller spiller = new Spiller();

                for (int i = 0; i < cfgs.length; ++i) {
                    LinearScan linScan = new LinearScan();
		    linScan.scan(cfgs[i]);
		    Frame frame = new Frame();
		    frame.rewriteCalls(cfgs[i]);
                    spiller.rewriteSpills(cfgs[i]);

                    String fileName = "cfg" + cfgs[i].name + ".dot";
                    PrintWriter writer =
                        new PrintWriter(new FileWriter(fileName));
                    printer.print(cfgs[i], writer);
                    writer.close();
                }

                Lineariser lin = new Lineariser();
                Code code = lin.lineariseProgram(analyzer.globalScope, cfgs);
                code.write(out);
            }
            out.close();
        } catch (IOException e) {
            Report.fail(e.getMessage());
        }
    }
}
