package miscc;

public class Opcode {
    // Values returned by defs/uses
    public static final int REG_NONE    = 0x00; // no registers
    public static final int REG_A       = 0x01; // register in "a" field
    public static final int REG_B       = 0x02; // register in "b" field
    public static final int REG_C       = 0x04; // register in "c" field
    public static final int REG_LNK     = 0x08; // LNK (aka R31)
    public static final int REG_UNKNOWN = 0x10; // don't know

    private static final int REG_AB = REG_A | REG_B;
    private static final int REG_AC = REG_A | REG_C;
    private static final int REG_BC = REG_B | REG_C;

    public final int format;
    public final int code;
    public final String mnemonic;
    public final int defs;
    public final int uses;
    public final int cost;
    public final boolean sideEffect;

    public Opcode(int format,
                  int code,
                  String mnemonic,
                  int defs,
                  int uses,
                  int cost,
                  boolean sideEffect) {
        this.format = format;
        this.code = code;
        this.mnemonic = mnemonic;
        this.defs = defs;
        this.uses = uses;
        this.cost = cost;
        this.sideEffect = sideEffect;
    }

    public Opcode(int format,
                  int code,
                  String mnemonic,
                  int defs,
                  int uses,
                  int cost) {
        this(format, code, mnemonic, defs, uses, cost, false);
    }

    public final static Opcode ADD =
        new Opcode(RISC.OpRaRbRc, RISC.ADD, "add", REG_A, REG_BC, 1);
    public final static Opcode SUB =
        new Opcode(RISC.OpRaRbRc, RISC.SUB, "sub", REG_A, REG_BC, 1);
    public final static Opcode MUL =
        new Opcode(RISC.OpRaRbRc, RISC.MUL, "mul", REG_A, REG_BC, 2);
    public final static Opcode DIV =
        new Opcode(RISC.OpRaRbRc, RISC.DIV, "div", REG_A, REG_BC, 2);
    public final static Opcode MOD =
        new Opcode(RISC.OpRaRbRc, RISC.MOD, "mod", REG_A, REG_BC, 2);
    public final static Opcode CMP =
        new Opcode(RISC.OpRaRbRc, RISC.CMP, "cmp", REG_A, REG_BC, 1);
    public final static Opcode OR =
        new Opcode(RISC.OpRaRbRc, RISC.OR, "or", REG_A, REG_BC, 1);
    public final static Opcode AND =
        new Opcode(RISC.OpRaRbRc, RISC.AND, "and", REG_A, REG_BC, 1);
    public final static Opcode BIC =
        new Opcode(RISC.OpRaRbRc, RISC.BIC, "bic", REG_A, REG_BC, 1);
    public final static Opcode XOR =
        new Opcode(RISC.OpRaRbRc, RISC.XOR, "xor", REG_A, REG_BC, 1);
    public final static Opcode LSH =
        new Opcode(RISC.OpRaRbRc, RISC.LSH, "lsh", REG_A, REG_BC, 1);
    public final static Opcode ASH =
        new Opcode(RISC.OpRaRbRc, RISC.ASH, "ash", REG_A, REG_BC, 1);

    public final static Opcode CHK =
        new Opcode(RISC.OpRa__Rc, RISC.CHK, "chk", REG_NONE, REG_AC, 1);

    public final static Opcode ADDI =
        new Opcode(RISC.OpRaRbIc, RISC.ADDI, "addi", REG_A, REG_B, 1);
    public final static Opcode SUBI =
        new Opcode(RISC.OpRaRbIc, RISC.SUBI, "subi", REG_A, REG_B, 1);
    public final static Opcode MULI =
        new Opcode(RISC.OpRaRbIc, RISC.MULI, "muli", REG_A, REG_B, 2);
    public final static Opcode DIVI =
        new Opcode(RISC.OpRaRbIc, RISC.DIVI, "divi", REG_A, REG_B, 2);
    public final static Opcode MODI =
        new Opcode(RISC.OpRaRbIc, RISC.MODI, "modi", REG_A, REG_B, 2);
    public final static Opcode CMPI =
        new Opcode(RISC.OpRaRbIc, RISC.CMPI, "cmpi", REG_A, REG_B, 1);
    public final static Opcode ORI =
        new Opcode(RISC.OpRaRbIc, RISC.ORI, "ori", REG_A, REG_B, 1);
    public final static Opcode ANDI =
        new Opcode(RISC.OpRaRbIc, RISC.ANDI, "andi", REG_A, REG_B, 1);
    public final static Opcode BICI =
        new Opcode(RISC.OpRaRbIc, RISC.BICI, "bici", REG_A, REG_B, 1);
    public final static Opcode XORI =
        new Opcode(RISC.OpRaRbIc, RISC.XORI, "xori", REG_A, REG_B, 1);
    public final static Opcode LSHI =
        new Opcode(RISC.OpRaRbIc, RISC.LSHI, "lshi", REG_A, REG_B, 1);
    public final static Opcode ASHI =
        new Opcode(RISC.OpRaRbIc, RISC.ASHI, "ashi", REG_A, REG_B, 1);

    public final static Opcode CHKI =
        new Opcode(RISC.OpRa__Ic, RISC.CHKI, "chki", REG_NONE, REG_A, 1);

    public final static Opcode ADDIU =
        new Opcode(RISC.OpRaRbUc, RISC.ADDIU, "addiu", REG_A, REG_B, 1);
    public final static Opcode SUBIU =
        new Opcode(RISC.OpRaRbUc, RISC.SUBIU, "subiu", REG_A, REG_B, 1);
    public final static Opcode MULIU =
        new Opcode(RISC.OpRaRbUc, RISC.MULIU, "muliu", REG_A, REG_B, 2);
    public final static Opcode DIVIU =
        new Opcode(RISC.OpRaRbUc, RISC.DIVIU, "diviu", REG_A, REG_B, 2);
    public final static Opcode MODIU =
        new Opcode(RISC.OpRaRbUc, RISC.MODIU, "modiu", REG_A, REG_B, 2);
    public final static Opcode CMPIU =
        new Opcode(RISC.OpRaRbUc, RISC.CMPIU, "cmpiu", REG_A, REG_B, 1);
    public final static Opcode ORIU =
        new Opcode(RISC.OpRaRbUc, RISC.ORIU, "oriu", REG_A, REG_B, 1);
    public final static Opcode ANDIU =
        new Opcode(RISC.OpRaRbUc, RISC.ANDIU, "andiu", REG_A, REG_B, 1);
    public final static Opcode BICIU =
        new Opcode(RISC.OpRaRbUc, RISC.BICIU, "biciu", REG_A, REG_B, 1);
    public final static Opcode XORIU =
        new Opcode(RISC.OpRaRbUc, RISC.XORIU, "xoriu", REG_A, REG_B, 1);

    public final static Opcode CHKIU =
        new Opcode(RISC.OpRa__Uc, RISC.CHKIU, "chkiu", REG_NONE, REG_A, 1);

    public final static Opcode LDW =
        new Opcode(RISC.OpRaRbIc, RISC.LDW, "ldw", REG_A, REG_B, 4);
    public final static Opcode LDB =
        new Opcode(RISC.OpRaRbIc, RISC.LDB, "ldb", REG_A, REG_B, 4);
    public final static Opcode POP =
        new Opcode(RISC.OpRaRbIc, RISC.POP, "pop", REG_AB, REG_B, 4, true);
    public final static Opcode STW =
        new Opcode(RISC.OpRaRbIc, RISC.STW, "stw", REG_NONE, REG_AB, 3, true);
    public final static Opcode STB =
        new Opcode(RISC.OpRaRbIc, RISC.STB, "stb", REG_NONE, REG_AB, 3, true);
    public final static Opcode PSH =
        new Opcode(RISC.OpRaRbIc, RISC.PSH, "psh", REG_B, REG_AB, 3, true);

    public final static Opcode BEQ =
        new Opcode(RISC.OpRa__Oc, RISC.BEQ, "beq", REG_NONE, REG_A, 2);
    public final static Opcode BNE =
        new Opcode(RISC.OpRa__Oc, RISC.BNE, "bne", REG_NONE, REG_A, 2);
    public final static Opcode BLT =
        new Opcode(RISC.OpRa__Oc, RISC.BLT, "blt", REG_NONE, REG_A, 2);
    public final static Opcode BGE =
        new Opcode(RISC.OpRa__Oc, RISC.BGE, "bge", REG_NONE, REG_A, 2);
    public final static Opcode BLE =
        new Opcode(RISC.OpRa__Oc, RISC.BLE, "ble", REG_NONE, REG_A, 2);
    public final static Opcode BGT =
        new Opcode(RISC.OpRa__Oc, RISC.BGT, "bgt", REG_NONE, REG_A, 2);

    public final static Opcode BSR =
        new Opcode(RISC.Op____Oc, RISC.BSR, "bsr", REG_LNK, REG_NONE, 2, true);
    public final static Opcode JSR =
        new Opcode(RISC.Op____Lc, RISC.JSR, "jsr", REG_LNK, REG_NONE, 2, true);
    public final static Opcode RET =
        new Opcode(RISC.Op____Rc, RISC.RET, "ret", REG_NONE, REG_C, 2, true);

    public final static Opcode BREAK =
        new Opcode(RISC.Op______, RISC.BREAK, "break", REG_NONE, REG_NONE, 0, true);

    public final static Opcode SYSCALL =
        new Opcode(RISC.OpRaRbUc, RISC.SYSCALL, "syscall", REG_UNKNOWN, REG_UNKNOWN, 10, true);

    public final static Opcode[] codes = new Opcode[255];
    static {
        codes[RISC.ADD]     = ADD;
        codes[RISC.SUB]     = SUB;
        codes[RISC.MUL]     = MUL;
        codes[RISC.DIV]     = DIV;
        codes[RISC.MOD]     = MOD;
        codes[RISC.CMP]     = CMP;
        codes[RISC.OR]      = OR;
        codes[RISC.AND]     = AND;
        codes[RISC.BIC]     = BIC;
        codes[RISC.XOR]     = XOR;
        codes[RISC.LSH]     = LSH;
        codes[RISC.ASH]     = ASH;
        codes[RISC.CHK]     = CHK;
        codes[RISC.ADDI]    = ADDI;
        codes[RISC.SUBI]    = SUBI;
        codes[RISC.MULI]    = MULI;
        codes[RISC.DIVI]    = DIVI;
        codes[RISC.MODI]    = MODI;
        codes[RISC.CMPI]    = CMPI;
        codes[RISC.ORI]     = ORI;
        codes[RISC.ANDI]    = ANDI;
        codes[RISC.BICI]    = BICI;
        codes[RISC.XORI]    = XORI;
        codes[RISC.LSHI]    = LSHI;
        codes[RISC.ASHI]    = ASHI;
        codes[RISC.CHKI]    = CHKI;
        codes[RISC.ADDIU]   = ADDIU;
        codes[RISC.SUBIU]   = SUBIU;
        codes[RISC.MULIU]   = MULIU;
        codes[RISC.DIVIU]   = DIVIU;
        codes[RISC.MODIU]   = MODIU;
        codes[RISC.CMPIU]   = CMPIU;
        codes[RISC.ORIU]    = ORIU;
        codes[RISC.ANDIU]   = ANDIU;
        codes[RISC.BICIU]   = BICIU;
        codes[RISC.XORIU]   = XORIU;
        codes[RISC.CHKIU]   = CHKIU;
        codes[RISC.LDW]     = LDW;
        codes[RISC.LDB]     = LDB;
        codes[RISC.POP]     = POP;
        codes[RISC.STW]     = STW;
        codes[RISC.STB]     = STB;
        codes[RISC.PSH]     = PSH;
        codes[RISC.BEQ]     = BEQ;
        codes[RISC.BNE]     = BNE;
        codes[RISC.BLT]     = BLT;
        codes[RISC.BGE]     = BGE;
        codes[RISC.BLE]     = BLE;
        codes[RISC.BGT]     = BGT;
        codes[RISC.BSR]     = BSR;
        codes[RISC.JSR]     = JSR;
        codes[RISC.RET]     = RET;
        codes[RISC.BREAK]   = BREAK;
        codes[RISC.SYSCALL] = SYSCALL;
    }
}
