/*  miscc -- a compiler for misc
 *    Source code position management
 *
 *    Matthias Zenger, Michel Schinz, Philippe Altherr
 *    $Id: Position.java,v 1.1 2004/04/02 14:44:27 schinz Exp $
 */

package miscc;


/** This class provides methods for encoding a location in
 *  a source file (line/column) into a single integer.
 */
final class Position {

    /** number of bits reserved for the column.
     */
    static final int columnBits = 12;

    private static final int columnMask = (1 << columnBits) - 1;

    /** undefined position
     */
    static final int UNDEFINED = 0;

    /** first position in a source file
     */
    static final int FIRST = (1 << columnBits) | 1;

    /** encode a line and column number into a single int
     */
    static int encode(int line, int col) {
        return (line << columnBits) | (col & columnMask);
    }

    /** get the line number out of an encoded position
     */
    static int line(int pos) {
        return pos >>> columnBits;
    }

    /** return the column number of an encoded position
     */
    static int column(int pos) {
        return pos & columnMask;
    }
}
