package miscc;

import java.util.BitSet;

public class RegisterPool {
    private int min, max;
    private BitSet used;

    public RegisterPool(int min, int max) {
        this.min = min;
        this.max = max;
        this.used = new BitSet();
    }

    public int get() {
        return get(used.nextClearBit(min));
    }

    public int get(int reg) {
        assert min <= reg && reg <= max
            : "attempt to allocate invalid register R" + reg;
        assert isFree(reg)
            : "attempt to allocate non-free register R" + reg;
        used.set(reg);
        return reg;
    }

    public void free(int reg) {
        assert !isFree(reg)
            : "attempt to free non-allocated register R" + reg;
        used.clear(reg);
    }

    public boolean isFree(int reg) {
        return !used.get(reg);
    }

    public BitSet used() {
        return (BitSet)used.clone();
    }

    public int usedCount() {
        return used.cardinality();
    }

    public int freeCount() {
        return (max - min + 1) - usedCount();
    }
}
