/*  miscc -- a compiler for misc
 *    Symbol table entries.
 *
 *    Matthias Zenger, Michel Schinz, Philippe Altherr
 *    $Id: Symbol.java,v 1.4 2005/01/17 09:54:32 schinz Exp $
 */
package miscc;


/** This class provides a representation for symbol table entries.
 */
abstract class Symbol {

    /** The position of the symbol's definition.
     */
    int pos;

    /** The name of the symbol.
     */
    String name;

    /** The type of the symbol.
     */
    Type type;

    /** The next symbol in the scope.
     */
    Symbol next;

    Symbol(int pos, String name, Type type) {
        this.pos = pos;
        this.name = name;
        this.type = type;
    }

    public String toString() {
        return name + ": " + type;
    }

    public abstract boolean isVariable();

    public static class Var extends Symbol {
        int offset;

        public Var(int pos, String name, Type type) {
            super(pos, name, type);
        }
        public boolean isVariable() { return true; }
    }

    public static class Fun extends Symbol {
        final String label;

        public Fun(int pos, String name, Type type, String label) {
            super(pos, name, type);
            this.label = label;
        }
        public boolean isVariable() { return false; }
    }
}
